@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Triangulation_2/include/CGAL/Constrained_Delaunay_triangulation_2.h $
+// $Id: Constrained_Delaunay_triangulation_2.h 630066d %aI Andreas Fabri
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Mariette Yvinec, Jean Daniel Boissonnat
@@ -78,6 +79,7 @@ public:
   typedef typename Ctr::Face_handle   Face_handle;
   typedef typename Ctr::Edge          Edge;
   typedef typename Ctr::Finite_faces_iterator Finite_faces_iterator;
+  typedef typename Ctr::Constrained_edges_iterator Constrained_edges_iterator;
   typedef typename Ctr::Face_circulator       Face_circulator;
   typedef typename Ctr::size_type             size_type;
   typedef typename Ctr::Locate_type           Locate_type;
@@ -89,6 +91,12 @@ public:
   typedef typename Ctr::Less_edge less_edge;
   typedef typename Ctr::Edge_set Edge_set;
 
+  //Tag to distinguish Delaunay from regular triangulations
+  typedef Tag_false Weighted_tag;
+
+  // Tag to distinguish periodic triangulations from others
+  typedef Tag_false Periodic_tag;
+
 #ifndef CGAL_CFG_USING_BASE_MEMBER_BUG_2
   using Ctr::geom_traits;
   using Ctr::number_of_vertices;
@@ -120,9 +128,6 @@ public:
   Constrained_Delaunay_triangulation_2(const Geom_traits& gt=Geom_traits()) 
     : Ctr(gt) { }
 
-  Constrained_Delaunay_triangulation_2(const CDt& cdt)
-    : Ctr(cdt) {}
-
   Constrained_Delaunay_triangulation_2(const List_constraints& lc,
 				       const Geom_traits& gt=Geom_traits())
     : Ctr(gt) 
@@ -742,9 +747,9 @@ propagating_flip(Face_handle f,int i, int depth)
 
   Face_handle ni = f->neighbor(i);
   flip(f, i); // flip for constrained triangulations
-  propagating_flip(f,i);
+  propagating_flip(f,i, depth+1);
   i = ni->index(f->vertex(i));
-  propagating_flip(ni,i);
+  propagating_flip(ni,i, depth+1);
 #endif
 }
 #else 
