@@ -8,8 +8,9 @@
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Number_types/include/CGAL/FPU_msvc.h $
+// $Id: FPU_msvc.h c651514 %aI Andreas Fabri
+// SPDX-License-Identifier: LGPL-3.0+
 //
 //
 // Author(s)     : Laurent Rineau
@@ -29,7 +30,7 @@ namespace CGAL {
 // replacement for C99 functions
 
 inline int
-feclearexcept(int exceptions) {
+feclearexcept(int /* exceptions */ ) {
   _clearfp();
   return 0;
 }
