@@ -13,8 +13,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Mesh_3/include/CGAL/Gray_image_mesh_domain_3.h $
+// $Id: Gray_image_mesh_domain_3.h 7ea3a80 %aI Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0+
 //
 //
@@ -31,25 +31,10 @@
 #include <CGAL/Random.h>
 #include <CGAL/Labeled_mesh_domain_3.h>
 #include <CGAL/Mesh_3/Image_to_labeled_function_wrapper.h>
-
-#include <functional>
+#include <CGAL/Bbox_3.h>
 
 namespace CGAL {
 
-namespace internal {
-
-  template<typename T>
-  struct Greater_than {
-    typedef T argument_type;
-    Greater_than(const T& second) : second(second) {}
-    bool operator()(const T& first) const {
-      return std::greater<T>()(first, second);
-    }
-    T second;
-  };
-
-}
-
 /**
  * @class Gray_image_mesh_domain_3
  *
@@ -57,26 +42,25 @@ namespace internal {
  */
 template<class Image,
          class BGT,
-         typename Image_word_type = float,
-         typename Transform = internal::Greater_than<double>,
+         typename Image_word_type_ = float,
+         typename Transform = Mesh_3::internal::Greater_than<double>,
          typename Subdomain_index = int>
-class Gray_image_mesh_domain_3
-  : public Labeled_mesh_domain_3<
-  Mesh_3::Image_to_labeled_function_wrapper<Image, BGT,
-                                            Image_word_type,
-                                            Subdomain_index,
-                                            Transform,
-                                            false> ,
-  BGT>
+class
+CGAL_DEPRECATED_MSG
+( "The class template `CGAL::Gray_image_mesh_domain_3` is now deprecated. "
+  "Use the static member function template "
+  "`Labeled_mesh_domain_3<K>::create_gray_image_mesh_domain` instead.")
+Gray_image_mesh_domain_3
+  : public Labeled_mesh_domain_3<BGT, Subdomain_index>
 {
 public:
-  typedef Mesh_3::Image_to_labeled_function_wrapper<Image, BGT,
-                                                    Image_word_type,
+  typedef Image_word_type_ Image_word_type;
+  typedef Mesh_3::Image_to_labeled_function_wrapper<Image_word_type,
+                                                    double,
                                                     Subdomain_index,
-                                                    Transform,
                                                     false>           Wrapper;
 
-  typedef Labeled_mesh_domain_3<Wrapper, BGT>                        Base;
+  typedef Labeled_mesh_domain_3<BGT, Subdomain_index>                Base;
 
   typedef typename Base::Sphere_3 Sphere_3;
   typedef typename Base::FT FT;
@@ -92,9 +76,8 @@ public:
     : Base(Wrapper(image, 
                    Transform(iso_value),
                    Transform(iso_value)(value_outside)),
-           compute_bounding_box(image),
+           Mesh_3::internal::compute_bounding_box(image),
            error_bound,
-           Null_subdomain_index(),
            p_rng)
   {
     CGAL_assertion(Transform(iso_value)(value_outside) == 0);
@@ -106,9 +89,8 @@ public:
                            const FT& error_bound = FT(1e-3),
                            CGAL::Random* p_rng = NULL)
     : Base(Wrapper(image, transform, transform(value_outside)),
-           compute_bounding_box(image),
+           Mesh_3::internal::compute_bounding_box(image),
            error_bound,
-           Null_subdomain_index(),
            p_rng)
   {
     CGAL_assertion(transform(value_outside) == 0);
@@ -116,30 +98,11 @@ public:
 
   /// Destructor
   virtual ~Gray_image_mesh_domain_3() {}
-
-
-private:
-  /// Returns a box enclosing image \c im
-  Bbox_3 compute_bounding_box(const Image& im) const
-  {
-    return Bbox_3(-1,-1,-1,
-                  double(im.xdim())*im.vx()+1,
-                  double(im.ydim())*im.vy()+1,
-                  double(im.zdim())*im.vz()+1);
-  }
-
-private:
-  // Disabled copy constructor & assignment operator
-  typedef Gray_image_mesh_domain_3<Image, BGT> Self;
-  Gray_image_mesh_domain_3(const Self& src);
-  Self& operator=(const Self& src);
-
 };  // end class Gray_image_mesh_domain_3
 
-
-
 }  // end namespace CGAL
 
 #include <CGAL/enable_warnings.h>
 
+
 #endif // CGAL_GRAY_IMAGE_MESH_DOMAIN_3_H
