@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Mesh_3/include/CGAL/Mesh_3/Mesh_global_optimizer.h $
+// $Id: Mesh_global_optimizer.h c28e05b %aI Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Stephane Tayeb
@@ -27,6 +28,7 @@
 
 #include <CGAL/license/Mesh_3.h>
 
+#include <CGAL/disable_warnings.h>
 
 #include <CGAL/Mesh_3/config.h>
 
@@ -36,6 +38,8 @@
 #include <CGAL/Origin.h>
 #include <CGAL/Mesh_optimization_return_code.h>
 #include <CGAL/Mesh_3/Null_global_optimizer_visitor.h>
+
+#include <CGAL/Hash_handles_with_or_without_timestamps.h>
 #include <CGAL/iterator.h>
 #include <CGAL/tuple.h>
 
@@ -74,11 +78,15 @@ class Mesh_global_optimizer_base
 protected:
   typedef typename Tr::Geom_traits                          Gt;
   typedef typename Gt::FT                                   FT;
+  typedef typename Gt::Vector_3                             Vector_3;
   typedef typename Tr::Lock_data_structure                  Lock_data_structure;
 
-  typedef std::vector<cpp11::tuple<
-    typename Tr::Vertex_handle, typename Tr::Point, FT> >   Moves_vector;
-  typedef unsigned int                                      Nb_frozen_points_type ;
+  // The sizing field info is stored inside the move vector because it is computed
+  // when the move is computed. This is because the parallel version uses the threadsafe
+  // version of incident_cells (which thus requires points to not be moving yet)
+  typedef std::vector<cpp11::tuple<typename Tr::Vertex_handle, Vector_3, FT> >
+                                                            Moves_vector;
+  typedef unsigned int                                      Nb_frozen_points_type;
 
   Mesh_global_optimizer_base(const Bbox_3 &, int)
     : big_moves_size_(0) {}
@@ -119,9 +127,10 @@ class Mesh_global_optimizer_base<Tr, Parallel_tag>
 protected:
   typedef typename Tr::Geom_traits                          Gt;
   typedef typename Gt::FT                                   FT;
+  typedef typename Gt::Vector_3                             Vector_3;
   typedef typename Tr::Lock_data_structure                  Lock_data_structure;
-  typedef tbb::concurrent_vector<cpp11::tuple<
-    typename Tr::Vertex_handle, typename Tr::Point, FT> >   Moves_vector;
+  typedef tbb::concurrent_vector<cpp11::tuple<typename Tr::Vertex_handle, Vector_3, FT> >
+                                                            Moves_vector;
   typedef tbb::atomic<unsigned int>                         Nb_frozen_points_type ;
 
   Mesh_global_optimizer_base(const Bbox_3 &bbox, int num_grid_cells_per_axis)
@@ -224,7 +233,8 @@ class Mesh_global_optimizer
   typedef typename C3T3::Triangulation  Tr;
   typedef typename Tr::Geom_traits      Gt;
 
-  typedef typename Tr::Point            Point_3;
+  typedef typename Tr::Bare_point       Bare_point;
+  typedef typename Tr::Weighted_point   Weighted_point;
   typedef typename Tr::Cell_handle      Cell_handle;
   typedef typename Tr::Vertex_handle    Vertex_handle;
   typedef typename Tr::Edge             Edge;
@@ -233,11 +243,12 @@ class Mesh_global_optimizer
   typedef typename Gt::FT               FT;
   typedef typename Gt::Vector_3         Vector_3;
 
-  typedef typename std::vector<Cell_handle>                 Cell_vector;
-  typedef typename std::vector<Vertex_handle>               Vertex_vector;
-  typedef typename std::set<Vertex_handle>                  Vertex_set;
-  typedef typename Base::Moves_vector                       Moves_vector;
-  typedef typename Base::Nb_frozen_points_type              Nb_frozen_points_type;
+  typedef typename std::vector<Cell_handle>                      Cell_vector;
+  typedef typename std::vector<Vertex_handle>                    Vertex_vector;
+  typedef Hash_handles_with_or_without_timestamps                Hash_fct;
+  typedef typename boost::unordered_set<Vertex_handle, Hash_fct> Vertex_set;
+  typedef typename Base::Moves_vector                            Moves_vector;
+  typedef typename Base::Nb_frozen_points_type                   Nb_frozen_points_type;
 
 #ifdef CGAL_INTRUSIVE_LIST
   typedef Intrusive_list<Cell_handle>   Outdated_cell_set;
@@ -296,95 +307,12 @@ private:
   /**
    * Returns moves for vertices of set \c moving_vertices
    */
-  Moves_vector compute_moves(Moving_vertices_set& moving_vertices)
-{
-  typename Gt::Construct_translated_point_3 translate =
-    Gt().construct_translated_point_3_object();
-
-  // Store new position of points which have to move
-  Moves_vector moves;
-
-
-  moves.reserve(moving_vertices.size());
-
-  // reset worst_move list
-  this->clear_big_moves();
-
-#ifdef CGAL_MESH_3_PROFILING
-  std::cerr << "Computing moves...";
-  WallClockTimer t;
-#endif
-
-
-#ifdef CGAL_LINKED_WITH_TBB
-  // Parallel
-  if (boost::is_convertible<Concurrency_tag, Parallel_tag>::value)
-  {
-    tbb::concurrent_vector<Vertex_handle> vertices_not_moving_any_more;
-
-    // Get move for each moving vertex
-    tbb::parallel_do(
-      moving_vertices.begin(), moving_vertices.end(),
-      Compute_move<Self, Sizing_field, Moves_vector,
-                   typename Gt::Construct_translated_point_3>(
-        *this, sizing_field_, moves, do_freeze_, vertices_not_moving_any_more,
-        translate)
-    );
-
-    typename tbb::concurrent_vector<Vertex_handle>::const_iterator it
-      = vertices_not_moving_any_more.begin();
-    typename tbb::concurrent_vector<Vertex_handle>::const_iterator it_end
-      = vertices_not_moving_any_more.end();
-    for ( ; it != it_end ; ++it)
-    {
-      moving_vertices.erase(*it);
-    }
-  }
-  // Sequential
-  else
-#endif // CGAL_LINKED_WITH_TBB
-  {
-    // Get move for each moving vertex
-    typename Moving_vertices_set::iterator vit = moving_vertices.begin();
-    for ( ; vit != moving_vertices.end() ; )
-    {
-      Vertex_handle oldv = *vit;
-      ++vit;
-      Vector_3 move = compute_move(oldv);
-
-      if ( CGAL::NULL_VECTOR != move )
-      {
-        Point_3 new_position = translate(oldv->point(),move);
-        FT size = (Sizing_field::is_vertex_update_needed ?
-          sizing_field_(new_position, oldv) : 0);
-        moves.push_back(cpp11::make_tuple(oldv,new_position,size));
-      }
-      else // CGAL::NULL_VECTOR == move
-      {
-        if(do_freeze_)
-          moving_vertices.erase(oldv); // TODO: if non-intrusive,
-                                       // we can optimize since we have the iterator,
-                                       // don't forget to do "vit = mv.erase(vit)" instead ++vit
-      }
-
-      // Stop if time_limit_ is reached
-      if ( is_time_limit_reached() )
-        break;
-    }
-  }
-
-#ifdef CGAL_MESH_3_PROFILING
-  std::cerr << "done in " << t.elapsed() << " seconds." << std::endl;
-#endif
-
-  return moves;
-}
-
+  Moves_vector compute_moves(Moving_vertices_set& moving_vertices);
 
   /**
    * Returns the move for vertex \c v
-   * warning : this function should be called only on moving vertices
-   *           even for frozen vertices, it could return a non-zero vector
+   * \warning This function should be called only on moving vertices
+   *          even for frozen vertices, it could return a non-zero vector
    */
   Vector_3 compute_move(const Vertex_handle& v);
 
@@ -402,7 +330,7 @@ private:
   void fill_sizing_field();
 
   /**
-   * Returns true if convergence is reached
+   * Returns `true` if convergence is reached
    */
   bool check_convergence() const;
 
@@ -423,7 +351,7 @@ private:
                             const Vertex_handle& v) const;
 
   /**
-   * Returns true if time_limit is reached
+   * Returns `true` if time_limit is reached
    */
   bool is_time_limit_reached() const
   {
@@ -434,8 +362,7 @@ private:
 
 #ifdef CGAL_LINKED_WITH_TBB
   // Functor for compute_moves function
-  template <typename MGO, typename Sizing_field_, typename Moves_vector_,
-            typename CTOP3>
+  template <typename MGO, typename Sizing_field_, typename Moves_vector_>
   class Compute_move
   {
     typedef tbb::concurrent_vector<Vertex_handle> Vertex_conc_vector;
@@ -445,49 +372,58 @@ private:
     Moves_vector_        & m_moves;
     bool                   m_do_freeze;
     Vertex_conc_vector   & m_vertices_not_moving_any_more;
-    const CTOP3          & m_translate;
+    const Gt             & m_gt;
 
   public:
     // Constructor
-    Compute_move(MGO &mgo, 
+    Compute_move(MGO &mgo,
                  const Sizing_field_ &sizing_field,
                  Moves_vector_ &moves,
                  bool do_freeze,
-                 Vertex_conc_vector & vertices_not_moving_any_more,
-                 const CTOP3 & translate)
+                 Vertex_conc_vector &vertices_not_moving_any_more,
+                 const Gt &gt)
     : m_mgo(mgo),
       m_sizing_field(sizing_field),
       m_moves(moves),
       m_do_freeze(do_freeze),
       m_vertices_not_moving_any_more(vertices_not_moving_any_more),
-      m_translate(translate)
+      m_gt(gt)
     {}
 
     // Constructor
     Compute_move(const Compute_move &cm)
-    : m_mgo(cm.m_mgo), 
-      m_sizing_field(cm.m_sizing_field), 
+    : m_mgo(cm.m_mgo),
+      m_sizing_field(cm.m_sizing_field),
       m_moves(cm.m_moves),
       m_do_freeze(cm.m_do_freeze),
       m_vertices_not_moving_any_more(cm.m_vertices_not_moving_any_more),
-      m_translate(cm.m_translate)
+      m_gt(cm.m_gt)
     {}
 
     // operator()
     void operator()(const Vertex_handle& oldv) const
     {
-      Vector_3 move = m_mgo.compute_move(oldv);
+      typename Gt::Construct_point_3 cp = m_gt.construct_point_3_object();
+      typename Gt::Construct_translated_point_3 translate = m_gt.construct_translated_point_3_object();
 
+      Vector_3 move = m_mgo.compute_move(oldv);
       if ( CGAL::NULL_VECTOR != move )
       {
-        Point_3 new_position = m_translate(oldv->point(), move);
-        FT size = (MGO::Sizing_field::is_vertex_update_needed ?
-          m_sizing_field(new_position, oldv) : 0);
+        FT size = 0.;
+
+        if(MGO::Sizing_field::is_vertex_update_needed)
+        {
+          const Weighted_point& position = m_mgo.tr_.point(oldv);
+          Bare_point new_position = translate(cp(position), move);
+          size = m_sizing_field(new_position, oldv);
+        }
+
         // typedef Triangulation_helpers<typename C3T3::Triangulation> Th;
         //if( !Th().inside_protecting_balls(tr_, oldv, new_position))
         //note : this is not happening for Lloyd and ODT so it's commented
         //       maybe for a new global optimizer it should be de-commented
-        m_moves.push_back(cpp11::make_tuple(oldv, new_position, size));
+
+        m_moves.push_back(cpp11::make_tuple(oldv, move, size));
       }
       else // CGAL::NULL_VECTOR == move
       {
@@ -503,33 +439,40 @@ private:
   };
 
   // Functor for fill_sizing_field function
-  template <typename MGO, typename Tr_, typename Local_list_>
+  template <typename MGO, typename Local_list_>
   class Compute_sizing_field_value
   {
     MGO                  & m_mgo;
+    const Gt             & m_gt;
     Local_list_          & m_local_lists;
 
   public:
     // Constructor
-    Compute_sizing_field_value(MGO &mgo, 
-                               Local_list_ & local_lists)
+    Compute_sizing_field_value(MGO &mgo,
+                               const Gt &gt,
+                               Local_list_ &local_lists)
     : m_mgo(mgo),
+      m_gt(gt),
       m_local_lists(local_lists)
     {}
 
     // Constructor
     Compute_sizing_field_value(const Compute_sizing_field_value &csfv)
-    : m_mgo(csfv.m_mgo), 
+    : m_mgo(csfv.m_mgo),
+      m_gt(csfv.m_gt),
       m_local_lists(csfv.m_local_lists)
     {}
 
     // operator()
     void operator()(Vertex& v) const
     {
-      Vertex_handle vh 
-        = Tr_::Triangulation_data_structure::Vertex_range::s_iterator_to(v);
+      typename Gt::Construct_point_3 cp = m_gt.construct_point_3_object();
+
+      Vertex_handle vh
+        = Tr::Triangulation_data_structure::Vertex_range::s_iterator_to(v);
+      const Weighted_point& position = m_mgo.tr_.point(vh);
       m_local_lists.local().push_back(
-        std::make_pair(v.point(), m_mgo.average_circumradius_length(vh)));
+          std::make_pair(cp(position), m_mgo.average_circumradius_length(vh)));
     }
   };
 
@@ -543,16 +486,16 @@ private:
     const Moves_vector_                  & m_moves;
     Moving_vertices_set_                 & m_moving_vertices;
     Outdated_cell_set_                   & m_outdated_cells;
-  
-    typedef typename Tr_::Point            Point_3;
-    typedef typename Tr_::Vertex_handle    Vertex_handle;
+
+    typedef typename Tr_::Bare_point    Bare_point;
+    typedef typename Tr_::Vertex_handle Vertex_handle;
 
   public:
     // Constructor
     Move_vertex(MGO &mgo, const Helper &helper, const Moves_vector_ &moves,
                 Moving_vertices_set_ &moving_vertices,
                 Outdated_cell_set_ &outdated_cells)
-    : m_mgo(mgo), m_helper(helper), m_moves(moves), 
+    : m_mgo(mgo), m_helper(helper), m_moves(moves),
       m_moving_vertices(moving_vertices), m_outdated_cells(outdated_cells)
     {}
 
@@ -569,21 +512,21 @@ private:
       for( size_t i = r.begin() ; i != r.end() ; ++i)
       {
         const Vertex_handle& v = cpp11::get<0>(m_moves[i]);
-        const Point_3& new_position = cpp11::get<1>(m_moves[i]);
+        const Vector_3& move = cpp11::get<1>(m_moves[i]);
+
         // Get size at new position
         if ( MGO::Sizing_field::is_vertex_update_needed )
         {
-          //FT size = sizing_field_(new_position,v);
           FT size = cpp11::get<2>(m_moves[i]);
 
           // Move point
           bool could_lock_zone;
           Vertex_handle new_v = m_helper.move_point(
-            v, new_position, m_outdated_cells, m_moving_vertices, &could_lock_zone);
+            v, move, m_outdated_cells, m_moving_vertices, &could_lock_zone);
           while (could_lock_zone == false)
           {
             new_v = m_helper.move_point(
-              v, new_position, m_outdated_cells, m_moving_vertices, &could_lock_zone);
+              v, move, m_outdated_cells, m_moving_vertices, &could_lock_zone);
           }
 
           // Restore size in meshing_info data
@@ -594,7 +537,7 @@ private:
           bool could_lock_zone;
           do {
             m_helper.move_point(
-              v, new_position, m_outdated_cells, m_moving_vertices, &could_lock_zone);
+              v, move, m_outdated_cells, m_moving_vertices, &could_lock_zone);
           } while (!could_lock_zone);
         }
 
@@ -655,9 +598,7 @@ Mesh_global_optimizer(C3T3& c3t3,
 , sizing_field_(c3t3.triangulation())
 , time_limit_(-1)
 , running_time_()
-
 , do_freeze_(do_freeze)
-
 #ifdef CGAL_MESH_3_OPTIMIZER_VERBOSE
 , sum_moves_(0)
 #endif // CGAL_MESH_3_OPTIMIZER_VERBOSE
@@ -681,7 +622,6 @@ Mesh_global_optimizer(C3T3& c3t3,
 }
 
 
-
 template <typename C3T3, typename Md, typename Mf, typename V_>
 Mesh_optimization_return_code
 Mesh_global_optimizer<C3T3,Md,Mf,V_>::
@@ -717,7 +657,7 @@ operator()(int nb_iterations, Visitor visitor)
   big_moves_size_ =
     (std::max)(std::size_t(1), std::size_t(moving_vertices.size()/500));
 
-  std::size_t nb_vertices_moved = -1;
+  std::size_t nb_vertices_moved = (std::numeric_limits<size_t>::max)();
   bool convergence_stop = false;
 
   // Iterate
@@ -733,14 +673,13 @@ operator()(int nb_iterations, Visitor visitor)
     Moves_vector moves = compute_moves(moving_vertices);
     visitor.after_compute_moves();
 
-    //Pb with Freeze : sometimes a few vertices continue moving indefinitely
-    //if the nb of moving vertices is < 1% of total nb AND does not decrease
+    // Pb with Freeze : sometimes a few vertices continue moving indefinitely
+    // if the nb of moving vertices is < 1% of total nb AND does not decrease
     if(do_freeze_
       && double(nb_vertices_moved) < 0.005 * double(initial_vertices_nb)
       && nb_vertices_moved == moving_vertices.size())
     {
-      // we should stop because we are
-      // probably entering an infinite instable loop
+      // we should stop because we are probably entering an infinite instable loop
       convergence_stop = true;
       break;
     }
@@ -821,6 +760,7 @@ operator()(int nb_iterations, Visitor visitor)
   return MAX_ITERATION_NUMBER_REACHED;
 }
 
+
 template <typename C3T3, typename Md, typename Mf, typename V_>
 void
 Mesh_global_optimizer<C3T3,Md,Mf,V_>::
@@ -831,20 +771,106 @@ collect_all_vertices(Moving_vertices_set& moving_vertices)
     moving_vertices.insert(vit);
 }
 
+template <typename C3T3, typename Md, typename Mf, typename V_>
+typename Mesh_global_optimizer<C3T3,Md,Mf,V_>::Moves_vector
+Mesh_global_optimizer<C3T3,Md,Mf,V_>::
+compute_moves(Moving_vertices_set& moving_vertices)
+{
+  // Store new position of points which have to move
+  Moves_vector moves;
+  moves.reserve(moving_vertices.size());
+
+  // reset worst_move list
+  this->clear_big_moves();
+
+#ifdef CGAL_MESH_3_PROFILING
+  std::cerr << "Computing moves...";
+  WallClockTimer t;
+#endif
+
+#ifdef CGAL_LINKED_WITH_TBB
+  // Parallel
+  if (boost::is_convertible<Concurrency_tag, Parallel_tag>::value)
+  {
+    tbb::concurrent_vector<Vertex_handle> vertices_not_moving_any_more;
+
+    // Get move for each moving vertex
+    tbb::parallel_do(
+          moving_vertices.begin(), moving_vertices.end(),
+          Compute_move<Self, Sizing_field, Moves_vector>(
+            *this, sizing_field_, moves, do_freeze_, vertices_not_moving_any_more,
+            tr_.geom_traits())
+          );
+
+    typename tbb::concurrent_vector<Vertex_handle>::const_iterator it
+        = vertices_not_moving_any_more.begin();
+    typename tbb::concurrent_vector<Vertex_handle>::const_iterator it_end
+        = vertices_not_moving_any_more.end();
+    for ( ; it != it_end ; ++it)
+    {
+      moving_vertices.erase(*it);
+    }
+  }
+  // Sequential
+  else
+#endif // CGAL_LINKED_WITH_TBB
+  {
+    typename Gt::Construct_point_3 cp = tr_.geom_traits().construct_point_3_object();
+    typename Gt::Construct_translated_point_3 translate = tr_.geom_traits().construct_translated_point_3_object();
+
+    // Get move for each moving vertex
+    typename Moving_vertices_set::iterator vit = moving_vertices.begin();
+    for ( ; vit != moving_vertices.end() ; )
+    {
+      Vertex_handle oldv = *vit;
+      ++vit;
+      Vector_3 move = compute_move(oldv);
+
+      if ( CGAL::NULL_VECTOR != move )
+      {
+        FT size = 0.;
+        if(Sizing_field::is_vertex_update_needed)
+        {
+          const Weighted_point& position = tr_.point(oldv);
+          Bare_point new_position = translate(cp(position), move);
+          size = sizing_field_(new_position, oldv);
+        }
+
+        moves.push_back(cpp11::make_tuple(oldv, move, size));
+      }
+      else // CGAL::NULL_VECTOR == move
+      {
+        if(do_freeze_)
+        {
+          // TODO: if non-intrusive, we can optimize since we have the iterator.
+          // Don't forget to do "vit = mv.erase(vit)" instead ++vit.
+          moving_vertices.erase(oldv);
+        }
+      }
+
+      // Stop if time_limit_ is reached
+      if ( is_time_limit_reached() )
+        break;
+    }
+  }
+
+#ifdef CGAL_MESH_3_PROFILING
+  std::cerr << "done in " << t.elapsed() << " seconds." << std::endl;
+#endif
+
+  return moves;
+}
+
 
 template <typename C3T3, typename Md, typename Mf, typename V_>
 typename Mesh_global_optimizer<C3T3,Md,Mf,V_>::Vector_3
 Mesh_global_optimizer<C3T3,Md,Mf,V_>::
 compute_move(const Vertex_handle& v)
 {
-  typename Gt::Compute_squared_length_3 sq_length =
-    Gt().compute_squared_length_3_object();
-
-  typename Gt::Construct_vector_3 vector =
-    Gt().construct_vector_3_object();
-
-  typename Gt::Construct_translated_point_3 translate =
-    Gt().construct_translated_point_3_object();
+  typename Gt::Construct_point_3 cp = tr_.geom_traits().construct_point_3_object();
+  typename Gt::Compute_squared_length_3 sq_length = tr_.geom_traits().compute_squared_length_3_object();
+  typename Gt::Construct_translated_point_3 translate = tr_.geom_traits().construct_translated_point_3_object();
+  typename Gt::Construct_vector_3 vector = tr_.geom_traits().construct_vector_3_object();
 
   Cell_vector incident_cells;
   incident_cells.reserve(64);
@@ -863,23 +889,25 @@ compute_move(const Vertex_handle& v)
   // Get move from move function
   Vector_3 move = move_function_(v, incident_cells, c3t3_, sizing_field_);
 
-  // Project surface vertex
-  if ( c3t3_.in_dimension(v) == 2 )
-  {
-    Point_3 new_position = translate(v->point(),move);
-    move = vector(v->point(), helper_.project_on_surface(new_position,v));
-  }
-
   FT local_sq_size = min_circumradius_sq_length(v, incident_cells);
   if ( FT(0) == local_sq_size )
     return CGAL::NULL_VECTOR;
 
+  // Project surface vertex
+  if ( c3t3_.in_dimension(v) == 2 )
+  {
+    const Weighted_point& position = tr_.point(v);
+    Bare_point new_position = translate(cp(position), move);
+    Bare_point projected_new_position = helper_.project_on_surface(v, new_position);
+    move = vector(cp(position), projected_new_position);
+  }
+
   FT local_move_sq_ratio = sq_length(move) / local_sq_size;
 
-  // Move point only if displacement is big enough w.r.t local size
+  // Move point only if the displacement is big enough w.r.t. the local size
   if ( local_move_sq_ratio < sq_freeze_ratio_ )
   {
-    nb_frozen_points_++;
+    ++nb_frozen_points_;
     return CGAL::NULL_VECTOR;
   }
 
@@ -927,22 +955,27 @@ update_mesh(const Moves_vector& moves,
          ++it )
     {
       const Vertex_handle& v = cpp11::get<0>(*it);
-      const Point_3& new_position = cpp11::get<1>(*it);
+      const Vector_3& move = cpp11::get<1>(*it);
       // Get size at new position
       if ( Sizing_field::is_vertex_update_needed )
       {
-        //FT size = sizing_field_(new_position,v);
         FT size = cpp11::get<2>(*it);
 
+#ifdef CGAL_MESH_3_OPTIMIZER_VERBOSE
+        std::cout << "Moving #" << it - moves.begin()
+                  << " addr: " << &*v
+                  << " pt: " << tr_.point(v)
+                  << " move: " << move << std::endl;
+#endif
         // Move point
-        Vertex_handle new_v = helper_.move_point(v, new_position, outdated_cells, moving_vertices);
+        Vertex_handle new_v = helper_.move_point(v, move, outdated_cells, moving_vertices);
 
         // Restore size in meshing_info data
         new_v->set_meshing_info(size);
       }
       else // Move point
       {
-        helper_.move_point(v, new_position, outdated_cells, moving_vertices);
+        helper_.move_point(v, move, outdated_cells, moving_vertices);
       }
 
       // Stop if time_limit_ is reached, here we can't return without rebuilding
@@ -952,7 +985,6 @@ update_mesh(const Moves_vector& moves,
     }
   }
 
-
   visitor.after_move_points();
 
 #ifdef CGAL_MESH_3_PROFILING
@@ -983,7 +1015,6 @@ update_mesh(const Moves_vector& moves,
 #ifdef CGAL_MESH_3_PROFILING
   std::cerr << "Updating C3T3 done in " << t.elapsed() << " seconds." << std::endl;
 #endif
-
 }
 
 
@@ -992,19 +1023,19 @@ void
 Mesh_global_optimizer<C3T3,Md,Mf,V_>::
 fill_sizing_field()
 {
-  std::map<Point_3,FT> value_map;
+  std::map<Bare_point,FT> value_map;
 
 #ifdef CGAL_LINKED_WITH_TBB
   // Parallel
   if (boost::is_convertible<Concurrency_tag, Parallel_tag>::value)
   {
     typedef tbb::enumerable_thread_specific<
-      std::vector< std::pair<Point_3, FT> > > Local_list;
+      std::vector< std::pair<Bare_point, FT> > > Local_list;
     Local_list local_lists;
 
     tbb::parallel_do(
       tr_.finite_vertices_begin(), tr_.finite_vertices_end(),
-      Compute_sizing_field_value<Self, Tr, Local_list>(*this, local_lists)
+      Compute_sizing_field_value<Self, Local_list>(*this, tr_.geom_traits(), local_lists)
     );
 
     for(typename Local_list::iterator it_list = local_lists.begin() ;
@@ -1017,17 +1048,19 @@ fill_sizing_field()
   else
 #endif //CGAL_LINKED_WITH_TBB
   {
+    typename Gt::Construct_point_3 cp = tr_.geom_traits().construct_point_3_object();
+
     // Fill map with local size
     for(typename Tr::Finite_vertices_iterator vit = tr_.finite_vertices_begin();
         vit != tr_.finite_vertices_end();
         ++vit)
     {
-      value_map.insert(std::make_pair(vit->point(),
-                                      average_circumradius_length(vit)));
+      const Weighted_point& position = tr_.point(vit);
+      value_map.insert(std::make_pair(cp(position), average_circumradius_length(vit)));
     }
   }
 
-  // fill sizing field
+  // Fill the sizing field
   sizing_field_.fill(value_map);
 }
 
@@ -1044,8 +1077,10 @@ check_convergence() const
     sum += CGAL::sqrt(*it);
   }
 
-  FT average_move = sum/FT(big_moves_size_);/*even if set is not full, divide*/
-       /*by max size so that if only 1 point moves, it goes to 0*/
+  // Even if set is not full, divide by the max size so that if only 1 point moves,
+  // then it goes to 0.
+  FT average_move = sum / FT(big_moves_size_);
+
 #ifdef CGAL_MESH_3_OPTIMIZER_VERBOSE
   sum_moves_ = average_move;
 #endif
@@ -1147,15 +1182,19 @@ typename Mesh_global_optimizer<C3T3,Md,Mf,V_>::FT
 Mesh_global_optimizer<C3T3,Md,Mf,V_>::
 sq_circumradius_length(const Cell_handle& cell, const Vertex_handle& v) const
 {
-  typename Gt::Compute_squared_distance_3 sq_distance =
-    Gt().compute_squared_distance_3_object();
+  typename Gt::Construct_point_3 cp = tr_.geom_traits().construct_point_3_object();
+  typename Gt::Compute_squared_distance_3 sq_distance = tr_.geom_traits().compute_squared_distance_3_object();
 
-  const Point_3 circumcenter = tr_.dual(cell);
-  return ( sq_distance(v->point(), circumcenter) );
+  const Bare_point circumcenter = tr_.dual(cell);
+  const Weighted_point& position = tr_.point(cell, cell->index(v));
+
+  return ( sq_distance(cp(position), circumcenter) );
 }
 
 } // end namespace Mesh_3
 
 } //namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_MESH_3_MESH_GLOBAL_OPTIMIZER_H
