@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Mesh_3/include/CGAL/Mesh_cell_criteria_3.h $
+// $Id: Mesh_cell_criteria_3.h 0fcc6c0 %aI Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Laurent RINEAU, Stephane Tayeb
@@ -36,7 +37,7 @@ class Mesh_cell_criteria_3
 public:
   typedef Visitor_ Visitor;
   typedef typename Visitor::Cell_quality Cell_quality;
-  typedef typename Visitor::Cell_badness Cell_badness;
+  typedef typename Visitor::Is_cell_bad  Is_cell_bad;
 
   typedef Mesh_3::Abstract_criterion<Tr,Visitor> Abstract_criterion;
 private:
@@ -81,13 +82,13 @@ public:
   ~Mesh_cell_criteria_3() { }
   
   /**
-   * @brief returns the badness of cell \c cell
+   * @brief returns whether the cell \c cell is bad or not.
+   * @param tr the triangulation within which \c cell lives
    * @param cell the cell
-   * @return the badness of \c cell
    */
-  Cell_badness operator()(const Cell_handle& cell) const
+  Is_cell_bad operator()(const Tr& tr, const Cell_handle& cell) const
   {
-    return criteria_(cell);
+    return criteria_(tr, cell);
   }
 
   void add(Abstract_criterion* criterion)
