@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Number_types/include/CGAL/Sqrt_extension/Sqrt_extension_type.h $
+// $Id: Sqrt_extension_type.h 1ec37a7 %aI Andreas Fabri
+// SPDX-License-Identifier: LGPL-3.0+
 //
 //
 // Author(s)     : Michael Hemmer   <hemmer@mpi-inf.mpg.de>
@@ -34,6 +35,8 @@
 #ifndef CGAL_SQRT_EXTENSION_TYPE_H
 #define CGAL_SQRT_EXTENSION_TYPE_H
 
+#include <CGAL/disable_warnings.h>
+
 #include <CGAL/number_type_basic.h>
 #include <boost/operators.hpp>
 #include <CGAL/Interval_arithmetic.h> 
@@ -150,13 +153,7 @@ public:
     explicit Sqrt_extension(const NTX& i)
         : a0_(NT(i)), a1_(NT(0)), root_(ROOT(0)), is_extended_(false) {}
 
-    /*! \brief copy constructor  */
-    Sqrt_extension(const Self& x)
-        : a0_(x.a0()),
-          a1_(x.a1()),
-          root_(x.root()),
-          is_extended_(x.is_extended()){}
-
+ 
 
     /*! \brief Expicite constructor of Sqrt_extension, from
      *  \c Sqrt_extension<NTX,ROOTX>.
@@ -170,11 +167,11 @@ public:
 //           is_extended_(x.is_extended()){}
 
     /*! \brief Constructor from some type NTX and ROOTX
-     *  NT must be constructible from NTX\\
+     *  NT must be constructible from NTX and NTY\\
      *  ROOT must be construcible from ROOTX\\
      */
-    template <class NTX,class ROOTX>
-    explicit Sqrt_extension(const NTX& a0, const NTX& a1, const ROOTX& root)
+    template <class NTX,class NTY,class ROOTX>
+    explicit Sqrt_extension(const NTX& a0, const NTY& a1, const ROOTX& root)
         : a0_(a0),
           a1_(a1),
           root_(root),
@@ -195,9 +192,10 @@ public:
     > >::type* = 0  )
   {
     typename Fraction_traits<NT>::Compose compose_nt;
+    typename Fraction_traits<ROOT>::Compose compose_root;
     if ( a != 0 ) {
       a0_ = compose_nt(-b,2*a);
-      root_ = CGAL_NTS square(a0_) - ROOT(c,a);
+      root_ = CGAL_NTS square(a0_) - compose_root(c,a);
       if(CGAL_NTS is_zero(root_)) {
 	is_extended_ = false;
       } else {
@@ -676,7 +674,7 @@ Sqrt_extension<NT,ROOT,ACDE_TAG,FP_TAG> square (const Sqrt_extension<NT,ROOT,ACD
 //NT_converter specializations
 template <class NT1,class ROOT1,class NT2,class ROOT2,class ACDE_TAG,class FP_TAG>
 struct NT_converter < Sqrt_extension<NT1,ROOT1,ACDE_TAG,FP_TAG> , Sqrt_extension<NT2,ROOT2,ACDE_TAG,FP_TAG> >
-  : public std::unary_function< NT1, NT2 >
+  : public CGAL::cpp98::unary_function< NT1, NT2 >
 {
     Sqrt_extension<NT2,ROOT2,ACDE_TAG,FP_TAG>
     operator()(const Sqrt_extension<NT1,ROOT1,ACDE_TAG,FP_TAG> &a) const
@@ -694,7 +692,7 @@ struct NT_converter < Sqrt_extension<NT1,ROOT1,ACDE_TAG,FP_TAG> , Sqrt_extension
 
 template <class NT1,class NT2,class ROOT2,class ACDE_TAG,class FP_TAG>
 struct NT_converter < NT1 , Sqrt_extension<NT2,ROOT2,ACDE_TAG,FP_TAG> >
-  : public std::unary_function< NT1, NT2 >
+  : public CGAL::cpp98::unary_function< NT1, NT2 >
 {
     Sqrt_extension<NT2,ROOT2,ACDE_TAG,FP_TAG>
     operator()(const NT1 &a) const
@@ -706,7 +704,7 @@ struct NT_converter < NT1 , Sqrt_extension<NT2,ROOT2,ACDE_TAG,FP_TAG> >
 //needed because it's a better match than the specialization <NT1,NT1>
 template <class NT1,class ROOT1,class ACDE_TAG,class FP_TAG>
 struct NT_converter < Sqrt_extension<NT1,ROOT1,ACDE_TAG,FP_TAG>, Sqrt_extension<NT1,ROOT1,ACDE_TAG,FP_TAG> >
-  : public std::unary_function< NT1, NT1 >
+  : public CGAL::cpp98::unary_function< NT1, NT1 >
 {
     const Sqrt_extension<NT1,ROOT1,ACDE_TAG,FP_TAG>&
     operator()(const Sqrt_extension<NT1,ROOT1,ACDE_TAG,FP_TAG> &a) const
@@ -824,6 +822,8 @@ print(std::ostream &os, const Sqrt_extension<NT_,ROOT_,ACDE_TAG_,FP_TAG> &r)
 
 #undef CGAL_int
 
+#include <CGAL/enable_warnings.h>
+
 #endif  // CGAL_SQRT_EXTENSION_TYPE_H
 
 // EOF
