@@ -13,8 +13,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Subdivision_method_3/include/CGAL/Subdivision_method_3/internal/subdivision_hosts_impl_3.h $
+// $Id: subdivision_hosts_impl_3.h 6dfa8d0 %aI Maxime Gimeno
 // SPDX-License-Identifier: LGPL-3.0+
 //
 //
@@ -173,7 +173,7 @@ void PQQ_1step(Poly& p, VertexPointMap vpm, Mask mask) {
   for (std::size_t i = 0; i < num_v; i++, ++vitr)
     put(vpm, *vitr, vertex_point_buffer[i]);
 
-//  CGAL_postcondition(p.is_valid());
+  CGAL_postcondition(CGAL::is_valid_polygon_mesh(p));
   delete []vertex_point_buffer;
 }
 
@@ -277,7 +277,7 @@ void PTQ_1step(Poly& p, VertexPointMap vpm, Mask mask) {
   for (std::size_t i = 0; i < num_v; i++, ++vitr)
     put(vpm, *vitr, vertex_point_buffer[i]);
 
-//  CGAL_postcondition(p.is_valid());
+  CGAL_postcondition(CGAL::is_valid_polygon_mesh(p));
   delete []vertex_point_buffer;
 }
 
@@ -527,7 +527,7 @@ void Sqrt3_1step(Poly& p, VertexPointMap vpm, Mask mask,
     }
   }
 
-//  CGAL_postcondition(p.is_valid());
+  CGAL_postcondition(CGAL::is_valid_polygon_mesh(p));
   delete []cpt;
 }
 
