@@ -12,14 +12,18 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Surface_mesher/include/CGAL/Surface_mesh_triangulation_generator_3.h $
+// $Id: Surface_mesh_triangulation_generator_3.h ee57fc2 %aI Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Laurent Rineau
 
 #ifndef CGAL_SURFACE_MESH_TRIANGULATION_GENERATOR_3_H
 #define CGAL_SURFACE_MESH_TRIANGULATION_GENERATOR_3_H
 
+#include <CGAL/license/Surface_mesher.h>
+
+
 #include <CGAL/Triangulation_data_structure_3.h>
 #include <CGAL/Delaunay_triangulation_3.h>
 #include <CGAL/Surface_mesh_vertex_base_3.h>
