@@ -11,8 +11,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Installation/include/CGAL/export/helpers.h $
+// $Id: helpers.h 0698f79 %aI Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Laurent Rineau
@@ -20,7 +21,7 @@
 #ifndef CGAL_EXPORT_HELPERS_H
 #define CGAL_EXPORT_HELPERS_H
 
-#ifdef CGAL_HEADER_ONLY
+#if defined(CGAL_HEADER_ONLY) && ! defined(CGAL_USE_Qt5_RESOURCES)
 #  define CGAL_DLL_IMPORT
 #  define CGAL_DLL_EXPORT
 #  define CGAL_DLL_LOCAL
