@@ -12,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Mesh_3/include/CGAL/exude_mesh_3.h $
+// $Id: exude_mesh_3.h 7ea3a80 %aI Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0+
 //
 //
@@ -34,8 +34,9 @@
 #include <CGAL/Mesh_3/Slivers_exuder.h>
 #include <CGAL/Mesh_optimization_return_code.h>
 #include <CGAL/Mesh_3/parameters_defaults.h>
-#include <CGAL/Mesh_3/global_parameters.h>
-#include <boost/parameter.hpp>
+#include <CGAL/boost/parameter.h>
+
+#include <boost/parameter/preprocessor.hpp>
 
 namespace CGAL {
 
@@ -46,8 +47,8 @@ namespace CGAL {
 
 // see <CGAL/config.h>
 CGAL_PRAGMA_DIAG_PUSH
-// see <CGAL/Mesh_3/config.h>
-CGAL_MESH_3_IGNORE_BOOST_PARAMETER_NAME_WARNINGS
+// see <CGAL/boost/parameter.h>
+CGAL_IGNORE_BOOST_PARAMETER_NAME_WARNINGS
 
 BOOST_PARAMETER_FUNCTION(
   (Mesh_optimization_return_code),
@@ -80,14 +81,14 @@ exude_mesh_3_impl(C3T3& c3t3,
   typedef Mesh_3::Min_dihedral_angle_criterion<Tr> Sc;
   //typedef Mesh_3::Radius_radio_criterion<Tr> Sc;
   typedef typename Mesh_3::Slivers_exuder<C3T3, Sc> Exuder;
-  
+
   // Create exuder
   Sc criterion(sliver_bound, c3t3.triangulation());
   Exuder exuder(c3t3, criterion);
 
   // Set time_limit
   exuder.set_time_limit(time_limit);
-  
+
   // Launch exudation
   return exuder();
 }
