@@ -12,6 +12,10 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Kernel_23/include/CGAL/internal/Projection_traits_3.h $
+// $Id: Projection_traits_3.h 9cc37e1 %aI Andreas Fabri
+// SPDX-License-Identifier: LGPL-3.0+
+//
 // Author(s)     : Mariette Yvinec, Sebastien Loriot, Mael Rouxel-Labbé
 
 #ifndef CGAL_INTERNAL_PROJECTION_TRAITS_3_H
@@ -244,6 +248,8 @@ public:
   typedef typename R::Point_2   Point_2; 
   typedef typename R::Line_3    Line_3; 
   typedef typename R::Line_2    Line_2;
+  typedef typename R::Segment_3 Segment_3;
+  typedef typename R::Segment_2 Segment_2;
   typedef typename R::FT        RT;
   typename R::FT x(const Point_3 &p) const { return Projector<R,dim>::x(p); }
   typename R::FT y(const Point_3 &p) const { return Projector<R,dim>::y(p); }
@@ -266,6 +272,13 @@ public:
     Line_2 l2(project(l.point(0)), project(l.point(1)));
     return squared_distance(p2, l2);
   }
+  
+  RT operator()(const Segment_3& s, const Point_3& p) const
+  {
+    Point_2 p2(project(p));
+    Segment_2 s2(project(s.source()), project(s.target()));
+    return squared_distance(p2, s2);
+  }
 };
 
 template <class R,int dim>
@@ -801,6 +814,7 @@ public:
   typedef typename Rp::Construct_scaled_vector_3              Construct_scaled_vector_2;
   typedef typename Rp::Construct_triangle_3                   Construct_triangle_2;
   typedef typename Rp::Construct_line_3                       Construct_line_2;
+  typedef typename Rp::Construct_bbox_3                       Construct_bbox_2;
 
   struct Less_xy_2 {
     typedef bool result_type;
@@ -985,6 +999,9 @@ public:
   Construct_line_2  construct_line_2_object() const
     {return Construct_line_2();}
 
+  Construct_bbox_2  construct_bbox_2_object() const
+    {return Construct_bbox_2();}
+
   Compute_scalar_product_2 compute_scalar_product_2_object() const
     {return Compute_scalar_product_2();}
 
